/*********************************
*     Memorization of spells     *
*********************************/

/*****************************************************************************+

A linked list takes care of the spells being memorized, then when it is
finished, it is saved using an array ch->player_specials->saved.spellmem[i].

Storage in playerfile:
ch->player_specials->saved.spellmem[i]

macros in utils.h:

GET_SPELLMEM(ch, i) returns the spell which is memorized in that slot.

defined in structs.h:

MAX_MEM is the total number of spell slots which can be held by a player.

void update_mem(void) is called in comm.c every tick to update the time
until a spell is memorized.

In spell_parser.c at the end of ACMD(do_cast), spell slots are checked when
a spell is cast, and the appropriate spell is removed from memory.

The number of available spell slots is determined by level and wisdom,

Expansion options:
Bard - compose
Priest - pray 
    
******************************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "db.h"
#include "interpreter.h"
#include "spells.h"
#include "handler.h"

#define SINFO spell_info[spellnum]
void update_mem(struct char_data *ch, bool mem_all);
int findslotnum(struct char_data *ch, int spelllvl);
int find_freeslot(struct char_data *ch, int spelllvl);
int find_memspeed(struct char_data *ch, bool display);

extern struct descriptor_data *descriptor_list;
extern struct char_data *is_playing(char *vict_name);
extern struct spell_info_type spell_info[];
extern struct char_data *character_list;

void memorize_remove(struct char_data * ch, struct memorize_node * mem);
void memorize_add(struct char_data * ch, int spellnum, int timer);

void displayslotnum(struct char_data *ch, int class);

void do_mem_display(struct char_data *ch)
{
  int speedfx, count, i, sortpos;
  struct memorize_node *mem, *next;
  
  strcpy(buf, "&cSpells in memory:\r\n&n");
  sprintf(buf + strlen(buf), "\r\n");
  strcpy(buf2, buf);

  /* List the memorized spells */
  for(i = 1; i < 9; i++)
  {    
    if((i >= 1) && (findslotnum(ch, i) < 1))
      break;
    sprintf(buf, "&c---&wLevel &R%d &wSpells&c---=============================&c---&w[&R%d &yslots&w]&c---&n\r\n\r\n", i, findslotnum(ch, i));
    count = 0;
    for(sortpos = 0; sortpos < GET_MEMCURSOR(ch); sortpos++)
    {
      if (strlen(buf2) >= MAX_STRING_LENGTH - 32) 
      {
	    strcat(buf2, "**OVERFLOW**\r\n");
	    break;
	  }
	  if ((GET_SPELLMEM(ch, sortpos) != 0) && (spell_info[GET_SPELLMEM(ch, sortpos)].spell_level == i))
	  {
	    count++;
	    sprintf(buf+strlen(buf), "&y%-22.22s&n", 
            spell_info[GET_SPELLMEM(ch,sortpos)].name);         
		if(count%3 == 0)
		  strcat(buf, "\r\n");
	  }
    }
    if(count%3 != 0)
      strcat(buf, "\r\n");
    sprintf(buf+strlen(buf), "&w(&r%d&y/&r%d&w)&n\r\n\r\n", count, findslotnum(ch, i));
    strcat(buf2, buf);
  }		

  /* here, list of spells being memorized and time till */
  /* memorization complete                              */ 
  speedfx = find_memspeed(ch, TRUE);

  sprintf(buf, "&c----------------------------------------------------------------&n\r\n");
  sprintf(buf + strlen(buf), "&cSpells being memorized:&n\r\n\r\n");
  count = 0;

  for (mem = ch->memorized; mem; mem = next) {
    next = mem->next;
    if ((mem->timer % speedfx) == 0) 
	  {
	    sprintf(buf+strlen(buf), "&y%-20.20s&w(&r%2dhr&w)&n ", spell_info[mem->spell].name, 
            (mem->timer/speedfx));
      }
	  else
      {
	    sprintf(buf+strlen(buf), "&y%-20.20s&w(&r%2dhr&w)&n ", spell_info[mem->spell].name, 
            ((mem->timer/speedfx)+1));
	  }
      count++;

	  if (count%2 == 0) 
        strcat(buf, "\r\n");
  }
  if(count%2 != 0)
    strcat(buf, "\r\n");
  if(count == 0)
    strcat(buf, "&w(&rNone&w)&n\r\n");
  strcat(buf2, buf);
  page_string(ch->desc, buf2, 1);
}


/*****************************************************************************+

   Memorization time for each spell is equal to the level of the spell + 2
   multiplied by 5 at POS_STANDING.  A level one spell would take 
   (1+2)*5=15 hours(ticks) to memorize when standing.

   POS_SITTING and POS_RESTING simulate the reduction in time by multiplying
   the number subtracted from the MEMTIME each tick by 5.  
   
   A character who has 15 hours(ticks) to memorize a spell standing will see
   this on his display.  When he is sitting, he will have 15 hours in MEMTIME,
   but the display will divide by the value returned in find_memspeed to show
   a value of 15/5 --> 3 hours sitting time.  

   If a tick occurs while sitting, update_mem will subtract 5 hours of 
   "standing time" which is one hour of "sitting time" from the timer.
   
******************************************************************************/
int find_memspeed(struct char_data *ch, bool display)
{
  int speedfx = 0; 

  if (GET_POS(ch) < POS_RESTING || GET_POS(ch) == POS_FIGHTING) {
    if(display)
      return 1;
    return speedfx;
  } else {
    if ((GET_POS(ch) == POS_RESTING) || (GET_POS(ch) == POS_SITTING))
      speedfx = /*int_app[GET_INT(ch)].memspeed +*/ 5;
    if (GET_POS(ch) == POS_STANDING)
      speedfx = /*int_app[GET_INT(ch)].memspeed +*/ 1;
    if (GET_COND(ch, DRUNK) > 10)
      speedfx = speedfx - 1;
    if (GET_COND(ch, FULL) == 0)
      speedfx = speedfx - 1;
    if (GET_COND(ch, THIRST) == 0)
      speedfx = speedfx - 1;
    speedfx = MAX(speedfx, 0);
    if(display)
      speedfx = MAX(speedfx, 1);
    return speedfx;
  }
}

/********************************************/
/* called during a tick to count down till  */ 
/* memorized in comm.c.                     */
/********************************************/

void update_mem(struct char_data *ch, bool mem_all)
{
  struct memorize_node *mem, *next_mem;
  struct descriptor_data *d;
  struct char_data *i;
  int speedfx = 0;

  for (d = descriptor_list; d; d = d->next) {
    if(ch)
      i = ch;
    else if(d->original)
      i = d->original;
    else if(!(i = d->character))
      continue;
    speedfx = find_memspeed(i, FALSE);
    for (mem = i->memorized; mem; mem = next_mem) {
      next_mem = mem->next;
      if (speedfx < mem->timer && !mem_all) {
        mem->timer -= speedfx;
      } else {
        send_to_char(i, "You have finished memorizing %s.\r\n", 
          spell_info[mem->spell].name);

        GET_SPELLMEM(i, GET_MEMCURSOR(i)) = mem->spell;
        GET_MEMCURSOR(i)++;

        memorize_remove(i, mem);
      }
    }
    if(ch)
      return;
  }
}

/* remove a spell from a character's memorize(in progress) linked list */
void memorize_remove(struct char_data * ch, struct memorize_node * mem)
{
  struct memorize_node *temp;

  if (ch->memorized == NULL) {
    core_dump();
    return;
  }

  REMOVE_FROM_LIST(mem, ch->memorized, next);
  free(mem);
}

/* add a spell to a character's memorize(in progress) linked list */
void memorize_add(struct char_data * ch, int spellnum, int timer)
{
  struct memorize_node * mem;

  CREATE(mem, struct memorize_node, 1);
  mem->timer = timer;
  mem->spell = spellnum;
  mem->next = ch->memorized;
  ch->memorized = mem;
}

/********************************************/
/*  type is forget, memorize, or stop       */
/*  message 0 for no message, 1 for message */
/********************************************/

void do_mem_spell(struct char_data *ch, char *arg, int type, int message)
{
  char *s;
  int spellnum, i;
  struct memorize_node *mem, *next;

  if (message == 1) {
    s = strtok(arg, "\0");
    if (s == NULL) {
      if (type == SCMD_MEMORIZE)
        send_to_char(ch, "Memorize what spell?!?\r\n");
      if (type == SCMD_STOP)
        send_to_char(ch, "Stop memorizing what spell?!?\r\n");
      if (type == SCMD_FORGET)
        send_to_char(ch, "Forget what spell?!?\r\n");
      return;
    }
    spellnum = find_skill_num(s);
  } else
    spellnum = atoi(arg);

  switch(type) {

/************* SCMD_MEMORIZE *************/

  case SCMD_MEMORIZE:
  if ((spellnum < 1) || (spellnum > MAX_SONGS)) {
    send_to_char(ch, "Memorize what?!?\r\n");
    return;
  }

  if (GET_SKILL(ch, spellnum) <= 0) {
    send_to_char(ch, "You are unfamiliar with that spell.\r\n");
    return;
  }

  /*if spell is practiced and there is an open spell slot*/
  if (find_freeslot(ch, spell_info[spellnum].spell_level) >= 1) {	  
    memorize_add(ch, spellnum, ((spell_info[spellnum].spell_level + 2) * 5));	
    if (message == 1) {
      send_to_char(ch, "You start memorizing %s.\r\n", spell_info[spellnum].name);
      return;
    }
  } else {
    if (message) 
      send_to_char(ch, "All of your level %d spell slots are currently filled\r\n", 
        spell_info[spellnum].spell_level);
    else
      /* if automem toggle is on */
      send_to_char(ch, "You cannot auto-rememorize because all of your level %d spell slots are currently filled.\r\n", 
        spell_info[spellnum].spell_level);
  }
  break;

/************* SCMD_STOP *************/

  case SCMD_STOP:
    if ((spellnum < 1) || (spellnum > MAX_SONGS)) {
      send_to_char(ch, "Stop memorizing what?!?\r\n");
      return;
    }
    
    for (mem = ch->memorized; mem; mem = next) {
      if (mem->spell == spellnum) {
        send_to_char(ch, "You stop memorizing %s.\r\n", spell_info[spellnum].name);
        memorize_remove(ch, mem);
        return;
      }
      next = mem->next;
    }

    send_to_char(ch, "%s is not being memorized.\r\n", spell_info[spellnum].name);
    break;

/************* SCMD_FORGET *************/

  case SCMD_FORGET:
    if ((spellnum < 1) || (spellnum > MAX_SONGS)) {
      send_to_char(ch, "Forget what?!?\r\n");
      return;
    }
    for (i = 0; i < GET_MEMCURSOR(ch); i++) {
      if(GET_SPELLMEM(ch, i) == spellnum) {
        GET_MEMCURSOR(ch)--;
        GET_SPELLMEM(ch, i) = GET_SPELLMEM(ch, GET_MEMCURSOR(ch));
	if (message) 
	  send_to_char(ch, "You forget the spell, %s.\r\n", spell_info[spellnum].name);
        return;
      }
    }
    send_to_char(ch, "%s is not memorized.\r\n", spell_info[spellnum].name);
    break;

/***********************************/

  }
}

/* returns the number of slots memorized if single is 0 
   returns 1 at the first occurance of the spell if memorized
   and single is 1 (true). 
*/
int is_memorized(struct char_data *ch, int spellnum, int single)
{
  int memcheck = 0, i;
  for (i = 0; i < GET_MEMCURSOR(ch); i++){
    if (GET_SPELLMEM(ch, i) == spellnum){
      memcheck++;
      if(single)
        return 1;
    }
  }
  return memcheck;
}

/**************************************************/
/* returns 1 if slot of spelllvl is open 0 if not */
/**************************************************/

int find_freeslot(struct char_data *ch, int spelllvl)
{
  struct memorize_node *mem, *next;
  int i, memcheck = 0;

  /* checked the memorized array */
  for (i = 0;i < GET_MEMCURSOR(ch); i++) {
    if (spell_info[GET_SPELLMEM(ch, i)].spell_level == spelllvl) {
      memcheck++;
    }
  }

  /* check the memorize linked list */
  for (mem = ch->memorized; mem; mem = next) {
    if (spell_info[mem->spell].spell_level == spelllvl) {  
      memcheck++;
    }
    next = mem->next;
  }

  if (memcheck < findslotnum(ch, spelllvl)) {
    return (1);
  } else {
    return 0;
  }
}

ACMD(do_memorize)
{

  if(IS_NPC(ch))
    return;

  one_argument(argument, arg); 

  switch (subcmd) {
  case SCMD_MEMORIZE:
    if (!*arg) {
      do_mem_display(ch);
    } else {
      if(!*argument)
        send_to_char(ch, "Memorize what spell?\r\n");
      else
        do_mem_spell(ch, argument, SCMD_MEMORIZE, 1);
    }
    break;
  case SCMD_STOP:
    if(!*argument) 
      send_to_char(ch, "Stop memorizing what?!?\r\n");
    else
      do_mem_spell(ch, argument, SCMD_STOP, 1);
    break;
  case SCMD_FORGET:
    if(!*argument) 
      send_to_char(ch, "Forget what spell?\r\n");
    else
      do_mem_spell(ch, argument, SCMD_FORGET, 1);
    break;
  case SCMD_WHEN_SLOT:
    displayslotnum(ch, GET_CLASS(ch));
    break;
  default:
    log("SYSERR: Unknown subcmd %d passed to do_memorize (%s)", subcmd, __FILE__);
    break;
  }
}

/************************************/
/*          Spell Levels            */
/*  1   2   3   4   5   6   7   8   */
/************************************/
  
          /* Arcanist */
int spell_lvlmax[(NUM_CLASSES * 31)][8] = {
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 1,  0,  0,  0,  0,  0,  0,  0 }, 
  { 2,  0,  0,  0,  0,  0,  0,  0 },
  { 2,  0,  0,  0,  0,  0,  0,  0 },
  { 3,  1,  0,  0,  0,  0,  0,  0 },
  { 3,  1,  0,  0,  0,  0,  0,  0 }, /* lvl 5  */
  { 3,  2,  1,  0,  0,  0,  0,  0 },
  { 4,  2,  1,  0,  0,  0,  0,  0 },
  { 4,  2,  1,  1,  0,  0,  0,  0 },
  { 4,  3,  2,  1,  0,  0,  0,  0 },
  { 4,  3,  2,  1,  0,  0,  0,  0 }, /* lvl 10 */
  { 5,  3,  2,  1,  1,  0,  0,  0 },
  { 5,  3,  3,  2,  1,  0,  0,  0 },
  { 5,  4,  3,  2,  1,  0,  0,  0 },
  { 5,  4,  3,  2,  1,  0,  0,  0 },
  { 5,  4,  3,  3,  2,  1,  0,  0 }, /* lvl 15 */
  { 5,  4,  4,  3,  2,  1,  0,  0 },
  { 5,  4,  4,  3,  2,  1,  0,  0 },
  { 5,  5,  4,  3,  3,  1,  0,  0 },
  { 5,  5,  4,  4,  3,  2,  0,  0 },
  { 5,  5,  4,  4,  3,  2,  1,  0 }, /* lvl 20 */
  { 5,  5,  5,  4,  3,  2,  1,  0 },
  { 5,  5,  5,  4,  4,  2,  1,  0 },
  { 5,  5,  5,  4,  4,  2,  1,  0 },
  { 5,  5,  5,  5,  4,  3,  1,  0 },
  { 5,  5,  5,  5,  4,  3,  2,  1 }, /* lvl 25 */
  { 5,  5,  5,  5,  4,  3,  2,  1 },
  { 5,  5,  5,  5,  4,  3,  2,  1 },
  { 5,  5,  5,  5,  4,  3,  2,  1 },
  { 5,  5,  5,  5,  4,  3,  3,  1 },
  { 5,  5,  5,  5,  4,  3,  3,  2 }, /* lvl 30 */

          /* Cleric */

  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 1,  0,  0,  0,  0,  0,  0,  0 }, 
  { 1,  0,  0,  0,  0,  0,  0,  0 },
  { 2,  0,  0,  0,  0,  0,  0,  0 },
  { 2,  1,  0,  0,  0,  0,  0,  0 },
  { 2,  1,  0,  0,  0,  0,  0,  0 }, /* lvl 5  */
  { 3,  2,  0,  0,  0,  0,  0,  0 },
  { 3,  2,  1,  0,  0,  0,  0,  0 },
  { 3,  2,  1,  0,  0,  0,  0,  0 },
  { 4,  3,  2,  0,  0,  0,  0,  0 },
  { 4,  3,  2,  1,  0,  0,  0,  0 }, /* lvl 10 */
  { 4,  3,  2,  1,  0,  0,  0,  0 },
  { 5,  3,  3,  2,  0,  0,  0,  0 },
  { 5,  4,  3,  2,  1,  0,  0,  0 },
  { 5,  4,  3,  2,  1,  0,  0,  0 },
  { 5,  4,  3,  3,  2,  0,  0,  0 }, /* lvl 15 */
  { 5,  4,  4,  3,  2,  1,  0,  0 },
  { 5,  4,  4,  3,  2,  1,  0,  0 },
  { 5,  5,  4,  3,  3,  2,  0,  0 },
  { 5,  5,  4,  4,  3,  2,  1,  0 },
  { 5,  5,  4,  4,  3,  2,  1,  0 }, /* lvl 20 */
  { 5,  5,  5,  4,  3,  3,  2,  0 },
  { 5,  5,  5,  4,  3,  3,  2,  0 },
  { 5,  5,  5,  5,  4,  3,  2,  0 },
  { 5,  5,  5,  5,  4,  3,  2,  0 },
  { 5,  5,  5,  5,  4,  4,  2,  0 }, /* lvl 25 */
  { 5,  5,  5,  5,  4,  4,  2,  0 },
  { 5,  5,  5,  5,  4,  4,  2,  0 },
  { 5,  5,  5,  5,  4,  4,  2,  0 },
  { 5,  5,  5,  5,  4,  4,  2,  0 },
  { 5,  5,  5,  5,  4,  4,  3,  0 }, /* lvl 30 */

          /* Rogue */

  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, 
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 5  */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 10 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 15 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 20 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 25 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 30 */

          /* Warrior */

  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, 
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 5  */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 10 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 15 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 20 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 25 */
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 },
  { 0,  0,  0,  0,  0,  0,  0,  0 }, /* lvl 30 */

};

/***********************************************************************/

/********************************/
/* bonus spells based on wisdom */
/********************************/

int spell_wisbonus[26][8] = {
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 0  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 1  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 2  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 3  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 4  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 5  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 6  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 7  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 8  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 9  */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 10 */
  { 0,  0,  0,  0,  0,  0,  0,  0  }, /* 11 */
  { 1,  0,  0,  0,  0,  0,  0,  0  }, /* 12 */
  { 1,  1,  0,  0,  0,  0,  0,  0  }, /* 13 */
  { 1,  1,  1,  0,  0,  0,  0,  0  }, /* 14 */
  { 1,  1,  1,  1,  0,  0,  0,  0  }, /* 15 */
  { 2,  1,  1,  1,  1,  0,  0,  0  }, /* 16 */
  { 2,  2,  1,  1,  1,  1,  0,  0  }, /* 17 */
  { 2,  2,  2,  1,  1,  1,  1,  0  }, /* 18 MAX_HUMAN NATURAL */
  { 2,  2,  2,  2,  1,  1,  1,  1  }, /* 19 */
  { 2,  2,  2,  2,  2,  1,  1,  1  }, /* 20 */
  { 2,  2,  2,  2,  2,  2,  1,  1  }, /* 21 */
  { 3,  2,  2,  2,  2,  2,  2,  1  }, /* 22 */
  { 3,  3,  2,  2,  2,  2,  2,  2  }, /* 23 */
  { 3,  3,  3,  2,  2,  2,  2,  2  }, /* 24 MAX_MORT */
  { 3,  3,  3,  3,  2,  2,  2,  2  }, /* 25 IMM-ONLY */
};         

/*****************************************************************/
/* Returns the number of spell slots for the character and spell */
/* level requested                                               */
/*****************************************************************/

int findslotnum(struct char_data *ch, int spelllvl)
{
  int spelllvl_num, spell_mod, stat_mod, slevel = 0;

  slevel = (GET_LEVEL(ch));
  slevel = MIN(slevel, 30);

  if (GET_LEVEL(ch) > 0)
    slevel = MAX(slevel, 1);

  slevel += (GET_CLASS(ch) * 31);

  /* check wisdom for bonus spell slots */
  stat_mod = spell_wisbonus[GET_WIS(ch)][spelllvl-1];

  spell_mod = spell_lvlmax[slevel][spelllvl-1];

  spelllvl_num = MAX(0, MIN(10, spell_mod + stat_mod)); 

   /* check to see if they have any slots for that level and type */
  if (!spell_lvlmax[slevel][spelllvl-1])
    spelllvl_num = 0; 

  /* spell level modifying EQ */
  spelllvl_num += GET_SPELL_LEVEL(ch, spelllvl);

  /* max of 10 slots per spell lvl */  
  spelllvl_num = MIN(10, spelllvl_num);

  return spelllvl_num;
}
